use std::str;
use curl::easy::{Easy, List};
use serde_json::json;
use serde_json::Value;
use regex::Regex;

use crate::common;
use crate::bar_modules;

#[derive(Clone)]
pub struct UnibarModuleWeather {
    opts:           common::AppOptions,
    weather_info:   Value,
}

impl UnibarModuleWeather {

    // --------------------
    pub fn new(o :common::AppOptions) -> Self {
        UnibarModuleWeather {
            opts: o,
            weather_info: json!(serde_json::Value::Null),
        }
    }

    // --------------------
    fn get_current_temperature(&self) -> f32 {
        let deg_c :f32 = self.weather_info["features"][0]["properties"]["temperature"]["value"]
                                    .to_string().parse().unwrap();

        match self.opts.weather_units {
            common::TemperatureUnits::Metric => return deg_c,
            common::TemperatureUnits::Imperial => return (deg_c * 9.0 / 5.0) + 32.0,
        }
    }

    // --------------------
    fn get_temperature_unit(&self) -> String{
        return if self.opts.weather_units == common::TemperatureUnits::Metric { "°C".to_string() }
                                                                         else { "°F".to_string() };
    }

    // --------------------
    fn get_unicode_symbol(&self, condition: &str) -> String {
        match condition {
            "skc"               => { return "☀️".to_string(); } // Fair/clear
            "few"               => { return "🌥️".to_string(); } // A few clouds
            "sct"               => { return "🌥️".to_string(); } // Partly cloudy
            "bkn"               => { return "🌥️".to_string(); } // Mostly cloudy
            "ovc"               => { return "☁️".to_string(); } // Overcast
            "wind_skc"          => { return "☀️".to_string(); } // Fair/clear and windy
            "wind_few"          => { return "🌥️".to_string(); } // A few clouds and windy
            "wind_sct"          => { return "🌥️".to_string(); } // Partly cloudy and windy
            "wind_bkn"          => { return "🌥️".to_string(); } // Mostly cloudy and windy
            "wind_ovc"          => { return "☁️".to_string(); } // Overcast and windy
            "snow"              => { return "❄️".to_string(); } // Snow
            "rain_snow"         => { return "🌨️".to_string(); } // Rain/snow
            "rain_sleet"        => { return "🌨️".to_string(); } // Rain/sleet
            "snow_sleet"        => { return "🌨️".to_string(); } // Snow/sleet
            "fzra"              => { return "🌨️".to_string(); } // Freezing rain
            "rain_fzra"         => { return "🌨️".to_string(); } // Rain/freezing rain
            "snow_fzra"         => { return "🌨️".to_string(); } // Freezing rain/snow
            "sleet"             => { return "🌨️".to_string(); } // Sleet
            "rain"              => { return "🌧️".to_string(); } // Rain
            "rain_showers"      => { return "🌧️".to_string(); } // Rain showers (high cloud cover)
            "rain_showers_hi"   => { return "🌧️".to_string(); } // Rain showers (low cloud cover)
            "tsra"              => { return "⛈️".to_string(); } // Thunderstorm (high cloud cover)
            "tsra_sct"          => { return "⛈️".to_string(); } // Thunderstorm (medium cloud cover)
            "tsra_hi"           => { return "⛈️".to_string(); } // Thunderstorm (low cloud cover)
            "tornado"           => { return "🌪️".to_string(); } // Tornado
            "hurricane"         => { return "🌬️".to_string(); } // Hurricane conditions
            "tropical_storm"    => { return "🌬️".to_string(); } // Tropical storm conditions
            "dust"              => { return "🌫️".to_string(); } // Dust
            "smoke"             => { return "🌫️".to_string(); } // Smoke
            "haze"              => { return "🌫️".to_string(); } // Haze
            "hot"               => { return "🥵".to_string(); } // Hot
            "cold"              => { return "🧣".to_string(); } // Cold
            "blizzard"          => { return "🥶".to_string(); } // Blizzard
            "fog"               => { return "🌫️".to_string(); } // Fog/mist
            _                   => { return "🌤️".to_string(); } // It is always sunny here
        }
    }

    // --------------------
    fn get_icons(&self) {
        // Print a web page onto stdout
        let mut curl        = Easy::new();
        let mut url_string  = Vec::new();
        let mut curl_ret    = Vec::new();

        url_string.push("https://api.weather.gov/icons".to_owned());

        curl.url(url_string.concat().as_str()).unwrap();
        {
            let mut list = List::new();
            list.append("User-Agent: Bar Weather (mahesh@heshapps.com)").unwrap();
            curl.http_headers(list).unwrap();

            let mut transfer = curl.transfer();

            transfer.write_function(|data| {
                curl_ret.extend_from_slice(data);
                Ok(data.len())
            }).unwrap();
            transfer.perform().unwrap();
        }
        println!("-----> curl_data - [{}]", std::str::from_utf8(&curl_ret).unwrap());
    }

    // --------------------
    fn show_icons(&self) {
        println!("{} skc             Fair/clear", self.get_unicode_symbol("skc"));
        println!("{} few             A few clouds", self.get_unicode_symbol("few"));
        println!("{} sct             Partly cloudy", self.get_unicode_symbol("sct"));
        println!("{} bkn             Mostly cloudy", self.get_unicode_symbol("bkn"));
        println!("{} ovc             Overcast", self.get_unicode_symbol("ovc"));
        println!("{} wind_skc        Fair/clear and windy", self.get_unicode_symbol("wind_skc"));
        println!("{} wind_few        A few clouds and windy", self.get_unicode_symbol("wind_few"));
        println!("{} wind_sct        Partly cloudy and windy", self.get_unicode_symbol("wind_sct"));
        println!("{} wind_bkn        Mostly cloudy and windy", self.get_unicode_symbol("wind_bkn"));
        println!("{} wind_ovc        Overcast and windy", self.get_unicode_symbol("wind_ovc"));
        println!("{} snow            Snow", self.get_unicode_symbol("snow"));
        println!("{} rain_snow       Rain/snow", self.get_unicode_symbol("rain_snow"));
        println!("{} rain_sleet      Rain/sleet", self.get_unicode_symbol("rain_sleet"));
        println!("{} snow_sleet      Snow/sleet", self.get_unicode_symbol("snow_sleet"));
        println!("{} fzra            Freezing rain", self.get_unicode_symbol("fzra"));
        println!("{} rain_fzra       Rain/freezing rain", self.get_unicode_symbol("rain_fzra"));
        println!("{} snow_fzra       Freezing rain/snow", self.get_unicode_symbol("snow_fzra"));
        println!("{} sleet           Sleet", self.get_unicode_symbol("sleet"));
        println!("{} rain            Rain", self.get_unicode_symbol("rain"));
        println!("{} rain_showers    Rain showers (high cloud cover)", self.get_unicode_symbol("rain_showers"));
        println!("{} rain_showers_hi Rain showers (low cloud cover)", self.get_unicode_symbol("rain_showers_hi"));
        println!("{} tsra            Thunderstorm (high cloud cover)", self.get_unicode_symbol("tsra"));
        println!("{} tsra_sct        Thunderstorm (medium cloud cover)", self.get_unicode_symbol("tsra_sct"));
        println!("{} tsra_hi         Thunderstorm (low cloud cover)", self.get_unicode_symbol("tsra_hi"));
        println!("{} tornado         Tornado", self.get_unicode_symbol("tornado"));
        println!("{} hurricane       Hurricane conditions", self.get_unicode_symbol("hurricane"));
        println!("{} tropical_storm  Tropical storm conditions", self.get_unicode_symbol("tropical_storm"));
        println!("{} dust            Dust", self.get_unicode_symbol("dust"));
        println!("{} smoke           Smoke", self.get_unicode_symbol("smoke"));
        println!("{} haze            Haze", self.get_unicode_symbol("haze"));
        println!("{} hot             Hot", self.get_unicode_symbol("hot"));
        println!("{} cold            Cold", self.get_unicode_symbol("cold"));
        println!("{} blizzard        Blizzard", self.get_unicode_symbol("blizzard"));
        println!("{} fog             Fog/mist", self.get_unicode_symbol("fog"));
        println!("{} _               It is always sunny here", self.get_unicode_symbol("_"));
    }

}

impl bar_modules::BarModuleActions for UnibarModuleWeather {

    // --------------------
    fn clear(&mut self) {
        self.weather_info = json!(serde_json::Value::Null);
    }

    // --------------------
    fn generate_data(&mut self) {
        // Print a web page onto stdout
        let mut curl        = Easy::new();
        let mut url_string  = Vec::new();
        let mut curl_ret    = Vec::new();

        url_string.push("https://api.weather.gov/stations/".to_owned());
        url_string.push(self.opts.weather_station.to_owned());
        url_string.push("/observations?limit=1".to_owned());

        curl.url(url_string.concat().as_str()).unwrap();
        {
            let mut list = List::new();
            list.append("User-Agent: Bar Weather (mahesh@heshapps.com)").unwrap();
            curl.http_headers(list).unwrap();

            let mut transfer = curl.transfer();

            transfer.write_function(|data| {
                curl_ret.extend_from_slice(data);
                Ok(data.len())
            }).unwrap();
            transfer.perform().unwrap();
        }
        if self.opts.debug {
            println!("-----> curl_data - [{}]", std::str::from_utf8(&curl_ret).unwrap());
        }

        self.weather_info = serde_json::from_str(str::from_utf8(&curl_ret).unwrap()).unwrap();
    }

    // --------------------
    fn get_content(&self) -> String {
        let temperature_value :f32 = self.get_current_temperature();
        let temperature_unit :String = self.get_temperature_unit();
        // let temperature_icon :String = self.get_icon(v.clone());

        return format!("{:.0}{}", temperature_value, temperature_unit);
    }

    // --------------------
    fn get_icon(&self) -> String {
        // "icon": "https://api.weather.gov/icons/land/night/ovc?size=medium",
        let re = Regex::new(r"(\w+)\?size").unwrap();
        let json_val = self.weather_info["features"][0]["properties"]["icon"].to_string();
        let caps = re.captures(&json_val).unwrap();

        return self.get_unicode_symbol(caps.get(1).unwrap().as_str());
    }
}

impl bar_modules::BarModuleDebug for UnibarModuleWeather {

    // --------------------
    fn post_debug(&self) {
        self.get_icons();
        self.show_icons();
    }
}
